{***************************************************************************}
{                             UNIDAD DE TECLADO

                                  Copyright (c) 1998 Marco Antonio Alvarado

Captura las teclas presionadas y liberadas, e informa su estado actual.

ADVERTENCIA

* Las teclas extendidas solo funcionan en los teclados extendidos, como los
  de 101 teclas.}
{***************************************************************************}

UNIT UTeclado;

INTERFACE

  VAR
    ViejaAtiendeActiva : Boolean;

{Estados de las teclas (True = presionada):
    1   ESC                                44   Z
    2   1                                  45   X
    3   2                                  46   C
    4   3                                  47   V
    5   4                                  48   B
    6   5                                  49   N
    7   6                                  50   M
    8   7                                  51   , <
    9   8                                  52   . >
   10   9                                  53   / ?
   11   0                                  54   RIGHT SHIFT
   12   - _                                55   *            (KEYPAD)
   13   = +                                56   LEFT ALT
   14   BACKSPACE                          57   SPACEBAR
   15   TAB                                58   CAPSLOCK
   16   Q                                  59   F1
   17   W                                  60   F2
   18   E                                  61   F3
   19   R                                  62   F4
   20   T                                  63   F5
   21   Y                                  64   F6
   22   U                                  65   F7
   23   I                                  66   F8
   24   O                                  67   F9
   25   P                                  68   F10
   26   [                                  69   NUMLOCK      (KEYPAD)
   27   ]                                  70   SCROLL LOCK
   28   ENTER (RETURN)                     71   7 HOME       (KEYPAD)
   29   LEFT CONTROL                       72   8 UP         (KEYPAD)
   30   A                                  73   9 PGUP       (KEYPAD)
   31   S                                  74   -            (KEYPAD)
   32   D                                  75   4 LEFT       (KEYPAD)
   33   F                                  76   5            (KEYPAD)
   34   G                                  77   6 RIGHT      (KEYPAD)
   35   H                                  78   +            (KEYPAD)
   36   J                                  79   1 END        (KEYPAD)
   37   K                                  80   2 DOWN       (KEYPAD)
   38   L                                  81   3 PGDN       (KEYPAD)
   39   ; :                                82   0 INSERT     (KEYPAD)
   40   ' "                                83   . DEL        (KEYPAD)
   41   ` ~                                87   F11
   42   LEFT SHIFT                         88   F12

Estados de las teclas extendidas:
   156  ENTER            (KEYPAD)          203  LEFT
   157  RIGHT CONTROL                      205  RIGHT
   170  PRINT SCREEN                       207  END
   181  /                (KEYPAD)          208  DOWN
   183  PRINT SCREEN                       209  PAGE DOWN
   184  RIGHT ALT                          210  INSERT
   199  HOME                               211  DELETE
   200  UP                                 239  MACRO
   201  PAGE UP}
    Teclas : ARRAY [Byte] OF Boolean;

IMPLEMENTATION

  USES
    UDS;  {Unidad de DOS Services}

  CONST
    InterrupcionTeclado = $09;

  VAR
    ViejaAtiende,
    ViejoExitProc : Pointer;
    Entrada : Byte;
    TeclaExtendida : Boolean;

{$F+}
  PROCEDURE Atiende; INTERRUPT;
{$F-}
  BEGIN
    Entrada := Port [96];

    IF Entrada = 224 THEN
    BEGIN
{El prximo cdigo ser una tecla extendida.}
      TeclaExtendida := True;
    END ELSE
    BEGIN

      IF TeclaExtendida THEN
      BEGIN
        TeclaExtendida := False;
{El binario ms significativo vale 1 si la tecla fue presionada y 0 si la
tecla fue liberada. Las teclas convencionales se almacenan en la coleccin
desde 0 hasta 127, mientras que las extendidas comienzan en 128.}
        Teclas [Entrada OR 128] := (Entrada AND 128) = 0;
      END ELSE
      BEGIN
        Teclas [Entrada AND 127] := (Entrada AND 128) = 0;
      END;

    END;

    IF ViejaAtiendeActiva THEN
    ASM
{Llama a la vieja interrupcin del teclado permitiendo que las rutinas que la
utilizan trabajen adecuadamente.}
      PUSHF
      CALL ViejaAtiende
    END ELSE
    BEGIN
{Reconoce la interrupcin.}
      Port [$20] := $20;
    END;

  END;

{$F+}
  PROCEDURE Sale;
{$F-}
  BEGIN
    ExitProc := ViejoExitProc;

    PoneInterrupcion (InterrupcionTeclado, ViejaAtiende);
  END;

BEGIN
  FillChar (Teclas, SizeOf (Teclas), False);
  TeclaExtendida := False;
  ViejaAtiendeActiva := False;
  ViejaAtiende := TomaInterrupcion (InterrupcionTeclado);
  PoneInterrupcion (InterrupcionTeclado, @Atiende);

  ViejoExitProc := ExitProc;
  ExitProc := @Sale;
END.